# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .shared.image_object import ImageObject
from .shared.followers_object import FollowersObject
from .shared.external_url_object import ExternalURLObject
from .shared.playlist_user_object import PlaylistUserObject
from .shared.playlist_track_object import PlaylistTrackObject

__all__ = ["PlaylistRetrieveResponse", "Owner", "Tracks"]


class Owner(PlaylistUserObject):
    """The user who owns the playlist"""

    display_name: Optional[str] = None
    """The name displayed on the user's profile. `null` if not available."""


class Tracks(BaseModel):
    """The tracks of the playlist."""

    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[PlaylistTrackObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class PlaylistRetrieveResponse(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    playlist.
    """

    collaborative: Optional[bool] = None
    """`true` if the owner allows other users to modify the playlist."""

    description: Optional[str] = None
    """The playlist description.

    _Only returned for modified, verified playlists, otherwise_ `null`.
    """

    external_urls: Optional[ExternalURLObject] = None
    """Known external URLs for this playlist."""

    followers: Optional[FollowersObject] = None
    """Information about the followers of the playlist."""

    href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the playlist."""

    images: Optional[List[ImageObject]] = None
    """Images for the playlist.

    The array may be empty or contain up to three images. The images are returned by
    size in descending order. See
    [Working with Playlists](/documentation/web-api/concepts/playlists). _**Note**:
    If returned, the source URL for the image (`url`) is temporary and will expire
    in less than a day._
    """

    name: Optional[str] = None
    """The name of the playlist."""

    owner: Optional[Owner] = None
    """The user who owns the playlist"""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    snapshot_id: Optional[str] = None
    """The version identifier for the current playlist.

    Can be supplied in other requests to target a specific playlist version
    """

    tracks: Optional[Tracks] = None
    """The tracks of the playlist."""

    type: Optional[str] = None
    """The object type: "playlist" """

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    playlist.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.me import (
    following_check_params,
    following_follow_params,
    following_unfollow_params,
    following_bulk_retrieve_params,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.me.following_check_response import FollowingCheckResponse
from ...types.me.following_bulk_retrieve_response import FollowingBulkRetrieveResponse

__all__ = ["FollowingResource", "AsyncFollowingResource"]


class FollowingResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FollowingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return FollowingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FollowingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return FollowingResourceWithStreamingResponse(self)

    def bulk_retrieve(
        self,
        *,
        type: Literal["artist"],
        after: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FollowingBulkRetrieveResponse:
        """
        Get the current user's followed artists.

        Args:
          type: The ID type: currently only `artist` is supported.

          after: The last artist ID retrieved from the previous request.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/me/following",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "type": type,
                        "after": after,
                        "limit": limit,
                    },
                    following_bulk_retrieve_params.FollowingBulkRetrieveParams,
                ),
            ),
            cast_to=FollowingBulkRetrieveResponse,
        )

    def check(
        self,
        *,
        ids: str,
        type: Literal["artist", "user"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FollowingCheckResponse:
        """
        Check to see if the current user is following one or more artists or other
        Spotify users.

        Args:
          ids: A comma-separated list of the artist or the user
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) to check. For
              example: `ids=74ASZWbe4lXaubB36ztrGX,08td7MxkoHQkXnWAYD8d6Q`. A maximum of 50
              IDs can be sent in one request.

          type: The ID type: either `artist` or `user`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/me/following/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "type": type,
                    },
                    following_check_params.FollowingCheckParams,
                ),
            ),
            cast_to=FollowingCheckResponse,
        )

    def follow(
        self,
        *,
        ids: SequenceNotStr[str],
        published: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Add the current user as a follower of one or more artists or other Spotify
        users.

        Args:
          ids: A JSON array of the artist or user
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `{ids:["74ASZWbe4lXaubB36ztrGX", "08td7MxkoHQkXnWAYD8d6Q"]}`. A maximum of 50
              IDs can be sent in one request. _**Note**: if the `ids` parameter is present in
              the query string, any IDs listed here in the body will be ignored._

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/following",
            body=maybe_transform(
                {
                    "ids": ids,
                    "published": published,
                },
                following_follow_params.FollowingFollowParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def unfollow(
        self,
        *,
        ids: SequenceNotStr[str] | Omit = omit,
        published: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove the current user as a follower of one or more artists or other Spotify
        users.

        Args:
          ids: A JSON array of the artist or user
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `{ids:["74ASZWbe4lXaubB36ztrGX", "08td7MxkoHQkXnWAYD8d6Q"]}`. A maximum of 50
              IDs can be sent in one request. _**Note**: if the `ids` parameter is present in
              the query string, any IDs listed here in the body will be ignored._

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            "/me/following",
            body=maybe_transform(
                {
                    "ids": ids,
                    "published": published,
                },
                following_unfollow_params.FollowingUnfollowParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncFollowingResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFollowingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncFollowingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFollowingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncFollowingResourceWithStreamingResponse(self)

    async def bulk_retrieve(
        self,
        *,
        type: Literal["artist"],
        after: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FollowingBulkRetrieveResponse:
        """
        Get the current user's followed artists.

        Args:
          type: The ID type: currently only `artist` is supported.

          after: The last artist ID retrieved from the previous request.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/me/following",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "type": type,
                        "after": after,
                        "limit": limit,
                    },
                    following_bulk_retrieve_params.FollowingBulkRetrieveParams,
                ),
            ),
            cast_to=FollowingBulkRetrieveResponse,
        )

    async def check(
        self,
        *,
        ids: str,
        type: Literal["artist", "user"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FollowingCheckResponse:
        """
        Check to see if the current user is following one or more artists or other
        Spotify users.

        Args:
          ids: A comma-separated list of the artist or the user
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) to check. For
              example: `ids=74ASZWbe4lXaubB36ztrGX,08td7MxkoHQkXnWAYD8d6Q`. A maximum of 50
              IDs can be sent in one request.

          type: The ID type: either `artist` or `user`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/me/following/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ids": ids,
                        "type": type,
                    },
                    following_check_params.FollowingCheckParams,
                ),
            ),
            cast_to=FollowingCheckResponse,
        )

    async def follow(
        self,
        *,
        ids: SequenceNotStr[str],
        published: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Add the current user as a follower of one or more artists or other Spotify
        users.

        Args:
          ids: A JSON array of the artist or user
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `{ids:["74ASZWbe4lXaubB36ztrGX", "08td7MxkoHQkXnWAYD8d6Q"]}`. A maximum of 50
              IDs can be sent in one request. _**Note**: if the `ids` parameter is present in
              the query string, any IDs listed here in the body will be ignored._

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/following",
            body=await async_maybe_transform(
                {
                    "ids": ids,
                    "published": published,
                },
                following_follow_params.FollowingFollowParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def unfollow(
        self,
        *,
        ids: SequenceNotStr[str] | Omit = omit,
        published: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove the current user as a follower of one or more artists or other Spotify
        users.

        Args:
          ids: A JSON array of the artist or user
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
              `{ids:["74ASZWbe4lXaubB36ztrGX", "08td7MxkoHQkXnWAYD8d6Q"]}`. A maximum of 50
              IDs can be sent in one request. _**Note**: if the `ids` parameter is present in
              the query string, any IDs listed here in the body will be ignored._

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            "/me/following",
            body=await async_maybe_transform(
                {
                    "ids": ids,
                    "published": published,
                },
                following_unfollow_params.FollowingUnfollowParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class FollowingResourceWithRawResponse:
    def __init__(self, following: FollowingResource) -> None:
        self._following = following

        self.bulk_retrieve = to_raw_response_wrapper(
            following.bulk_retrieve,
        )
        self.check = to_raw_response_wrapper(
            following.check,
        )
        self.follow = to_raw_response_wrapper(
            following.follow,
        )
        self.unfollow = to_raw_response_wrapper(
            following.unfollow,
        )


class AsyncFollowingResourceWithRawResponse:
    def __init__(self, following: AsyncFollowingResource) -> None:
        self._following = following

        self.bulk_retrieve = async_to_raw_response_wrapper(
            following.bulk_retrieve,
        )
        self.check = async_to_raw_response_wrapper(
            following.check,
        )
        self.follow = async_to_raw_response_wrapper(
            following.follow,
        )
        self.unfollow = async_to_raw_response_wrapper(
            following.unfollow,
        )


class FollowingResourceWithStreamingResponse:
    def __init__(self, following: FollowingResource) -> None:
        self._following = following

        self.bulk_retrieve = to_streamed_response_wrapper(
            following.bulk_retrieve,
        )
        self.check = to_streamed_response_wrapper(
            following.check,
        )
        self.follow = to_streamed_response_wrapper(
            following.follow,
        )
        self.unfollow = to_streamed_response_wrapper(
            following.unfollow,
        )


class AsyncFollowingResourceWithStreamingResponse:
    def __init__(self, following: AsyncFollowingResource) -> None:
        self._following = following

        self.bulk_retrieve = async_to_streamed_response_wrapper(
            following.bulk_retrieve,
        )
        self.check = async_to_streamed_response_wrapper(
            following.check,
        )
        self.follow = async_to_streamed_response_wrapper(
            following.follow,
        )
        self.unfollow = async_to_streamed_response_wrapper(
            following.unfollow,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["PlayerTransferParams"]


class PlayerTransferParams(TypedDict, total=False):
    device_ids: Required[SequenceNotStr[str]]
    """
    A JSON array containing the ID of the device on which playback should be
    started/transferred.<br/>For
    example:`{device_ids:["74ASZWbe4lXaubB36ztrGX"]}`<br/>_**Note**: Although an
    array is accepted, only a single device_id is currently supported. Supplying
    more than one will return `400 Bad Request`_
    """

    play: bool
    """
    **true**: ensure playback happens on new device.<br/>**false** or not provided:
    keep the current playback state.
    """

    published: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PlayerListRecentlyPlayedParams"]


class PlayerListRecentlyPlayedParams(TypedDict, total=False):
    after: int
    """A Unix timestamp in milliseconds.

    Returns all items after (but not including) this cursor position. If `after` is
    specified, `before` must not be specified.
    """

    before: int
    """A Unix timestamp in milliseconds.

    Returns all items before (but not including) this cursor position. If `before`
    is specified, `after` must not be specified.
    """

    limit: int
    """The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CategoryRetrieveParams"]


class CategoryRetrieveParams(TypedDict, total=False):
    locale: str
    """
    The desired language, consisting of an
    [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
    [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
    joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
    (Mexico)&quot;. Provide this parameter if you want the category strings returned
    in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
    specified language is not available, the category strings returned will be in
    the Spotify default language (American English)._
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["TrackRemoveParams", "Track"]


class TrackRemoveParams(TypedDict, total=False):
    tracks: Required[Iterable[Track]]
    """
    An array of objects containing
    [Spotify URIs](/documentation/web-api/concepts/spotify-uris-ids) of the tracks
    or episodes to remove. For example:
    `{ "tracks": [{ "uri": "spotify:track:4iV5W9uYEdYUVa79Axb7Rh" },{ "uri": "spotify:track:1301WleyT98MSxVHPZCA6M" }] }`.
    A maximum of 100 objects can be sent at once.
    """

    published: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    snapshot_id: str
    """
    The playlist's snapshot ID against which you want to make the changes. The API
    will validate that the specified items exist and in the specified positions and
    make the changes, even if more recent changes have been made to the playlist.
    """


class Track(TypedDict, total=False):
    uri: str
    """Spotify URI"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ..types import audio_feature_bulk_retrieve_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.audio_feature_retrieve_response import AudioFeatureRetrieveResponse
from ..types.audio_feature_bulk_retrieve_response import AudioFeatureBulkRetrieveResponse

__all__ = ["AudioFeaturesResource", "AsyncAudioFeaturesResource"]


class AudioFeaturesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AudioFeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AudioFeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AudioFeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AudioFeaturesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioFeatureRetrieveResponse:
        """
        Get audio feature information for a single track identified by its unique
        Spotify ID.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              track.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/audio-features/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AudioFeatureRetrieveResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def bulk_retrieve(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioFeatureBulkRetrieveResponse:
        """
        Get audio features for multiple tracks based on their Spotify IDs.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the tracks.
              Maximum: 100 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/audio-features",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, audio_feature_bulk_retrieve_params.AudioFeatureBulkRetrieveParams),
            ),
            cast_to=AudioFeatureBulkRetrieveResponse,
        )


class AsyncAudioFeaturesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAudioFeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncAudioFeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAudioFeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncAudioFeaturesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioFeatureRetrieveResponse:
        """
        Get audio feature information for a single track identified by its unique
        Spotify ID.

        Args:
          id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
              track.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/audio-features/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AudioFeatureRetrieveResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def bulk_retrieve(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioFeatureBulkRetrieveResponse:
        """
        Get audio features for multiple tracks based on their Spotify IDs.

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the tracks.
              Maximum: 100 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/audio-features",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"ids": ids}, audio_feature_bulk_retrieve_params.AudioFeatureBulkRetrieveParams
                ),
            ),
            cast_to=AudioFeatureBulkRetrieveResponse,
        )


class AudioFeaturesResourceWithRawResponse:
    def __init__(self, audio_features: AudioFeaturesResource) -> None:
        self._audio_features = audio_features

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                audio_features.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )
        self.bulk_retrieve = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                audio_features.bulk_retrieve,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncAudioFeaturesResourceWithRawResponse:
    def __init__(self, audio_features: AsyncAudioFeaturesResource) -> None:
        self._audio_features = audio_features

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                audio_features.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )
        self.bulk_retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                audio_features.bulk_retrieve,  # pyright: ignore[reportDeprecated],
            )
        )


class AudioFeaturesResourceWithStreamingResponse:
    def __init__(self, audio_features: AudioFeaturesResource) -> None:
        self._audio_features = audio_features

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                audio_features.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )
        self.bulk_retrieve = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                audio_features.bulk_retrieve,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncAudioFeaturesResourceWithStreamingResponse:
    def __init__(self, audio_features: AsyncAudioFeaturesResource) -> None:
        self._audio_features = audio_features

        self.retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                audio_features.retrieve,  # pyright: ignore[reportDeprecated],
            )
        )
        self.bulk_retrieve = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                audio_features.bulk_retrieve,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .browse import (
    BrowseResource,
    AsyncBrowseResource,
    BrowseResourceWithRawResponse,
    AsyncBrowseResourceWithRawResponse,
    BrowseResourceWithStreamingResponse,
    AsyncBrowseResourceWithStreamingResponse,
)
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)

__all__ = [
    "CategoriesResource",
    "AsyncCategoriesResource",
    "CategoriesResourceWithRawResponse",
    "AsyncCategoriesResourceWithRawResponse",
    "CategoriesResourceWithStreamingResponse",
    "AsyncCategoriesResourceWithStreamingResponse",
    "BrowseResource",
    "AsyncBrowseResource",
    "BrowseResourceWithRawResponse",
    "AsyncBrowseResourceWithRawResponse",
    "BrowseResourceWithStreamingResponse",
    "AsyncBrowseResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PlaylistCreateParams"]


class PlaylistCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name for the new playlist, for example `"Your Coolest Playlist"`.

    This name does not need to be unique; a user may have several playlists with the
    same name.
    """

    collaborative: bool
    """Defaults to `false`.

    If `true` the playlist will be collaborative. _**Note**: to create a
    collaborative playlist you must also set `public` to `false`. To create
    collaborative playlists you must have granted `playlist-modify-private` and
    `playlist-modify-public`
    [scopes](/documentation/web-api/concepts/scopes/#list-of-scopes)._
    """

    description: str
    """
    value for playlist description as displayed in Spotify Clients and in the Web
    API.
    """

    published: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["TrackSaveParams", "TimestampedID"]


class TrackSaveParams(TypedDict, total=False):
    ids: Required[SequenceNotStr[str]]
    """
    A JSON array of the
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
    `["4iV5W9uYEdYUVa79Axb7Rh", "1301WleyT98MSxVHPZCA6M"]`<br/>A maximum of 50 items
    can be specified in one request. _**Note**: if the `timestamped_ids` is present
    in the body, any IDs listed in the query parameters (deprecated) or the `ids`
    field in the body will be ignored._
    """

    published: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    timestamped_ids: Iterable[TimestampedID]
    """A JSON array of objects containing track IDs with their corresponding
    timestamps.

    Each object must include a track ID and an `added_at` timestamp. This allows you
    to specify when tracks were added to maintain a specific chronological order in
    the user's library.<br/>A maximum of 50 items can be specified in one request.
    _**Note**: if the `timestamped_ids` is present in the body, any IDs listed in
    the query parameters (deprecated) or the `ids` field in the body will be
    ignored._
    """


class TimestampedID(TypedDict, total=False):
    id: Required[str]
    """
    The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    track.
    """

    added_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The timestamp when the track was added to the library.

    Use ISO 8601 format with UTC timezone (e.g., `2023-01-15T14:30:00Z`). You can
    specify past timestamps to insert tracks at specific positions in the library's
    chronological order. The API uses minute-level granularity for ordering, though
    the timestamp supports millisecond precision.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorURLPage, AsyncCursorURLPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.browse import category_list_params, category_retrieve_params, category_get_playlists_params
from ...types.browse.category_list_response import CategoryListResponse
from ...types.browse.category_retrieve_response import CategoryRetrieveResponse
from ...types.browse.category_get_playlists_response import CategoryGetPlaylistsResponse

__all__ = ["CategoriesResource", "AsyncCategoriesResource"]


class CategoriesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CategoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return CategoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CategoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return CategoriesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        category_id: str,
        *,
        locale: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CategoryRetrieveResponse:
        """
        Get a single category used to tag items in Spotify (on, for example, the Spotify
        player’s “Browse” tab).

        Args:
          category_id: The [Spotify category ID](/documentation/web-api/concepts/spotify-uris-ids) for
              the category.

          locale: The desired language, consisting of an
              [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
              [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
              joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
              (Mexico)&quot;. Provide this parameter if you want the category strings returned
              in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
              specified language is not available, the category strings returned will be in
              the Spotify default language (American English)._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._get(
            f"/browse/categories/{category_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"locale": locale}, category_retrieve_params.CategoryRetrieveParams),
            ),
            cast_to=CategoryRetrieveResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        locale: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[CategoryListResponse]:
        """
        Get a list of categories used to tag items in Spotify (on, for example, the
        Spotify player’s “Browse” tab).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          locale: The desired language, consisting of an
              [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
              [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
              joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
              (Mexico)&quot;. Provide this parameter if you want the category strings returned
              in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
              specified language is not available, the category strings returned will be in
              the Spotify default language (American English)._

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/browse/categories",
            page=SyncCursorURLPage[CategoryListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "locale": locale,
                        "offset": offset,
                    },
                    category_list_params.CategoryListParams,
                ),
            ),
            model=CategoryListResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def get_playlists(
        self,
        category_id: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CategoryGetPlaylistsResponse:
        """
        Get a list of Spotify playlists tagged with a particular category.

        Args:
          category_id: The [Spotify category ID](/documentation/web-api/concepts/spotify-uris-ids) for
              the category.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._get(
            f"/browse/categories/{category_id}/playlists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    category_get_playlists_params.CategoryGetPlaylistsParams,
                ),
            ),
            cast_to=CategoryGetPlaylistsResponse,
        )


class AsyncCategoriesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCategoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncCategoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCategoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncCategoriesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        category_id: str,
        *,
        locale: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CategoryRetrieveResponse:
        """
        Get a single category used to tag items in Spotify (on, for example, the Spotify
        player’s “Browse” tab).

        Args:
          category_id: The [Spotify category ID](/documentation/web-api/concepts/spotify-uris-ids) for
              the category.

          locale: The desired language, consisting of an
              [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
              [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
              joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
              (Mexico)&quot;. Provide this parameter if you want the category strings returned
              in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
              specified language is not available, the category strings returned will be in
              the Spotify default language (American English)._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._get(
            f"/browse/categories/{category_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"locale": locale}, category_retrieve_params.CategoryRetrieveParams),
            ),
            cast_to=CategoryRetrieveResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        locale: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CategoryListResponse, AsyncCursorURLPage[CategoryListResponse]]:
        """
        Get a list of categories used to tag items in Spotify (on, for example, the
        Spotify player’s “Browse” tab).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          locale: The desired language, consisting of an
              [ISO 639-1](http://en.wikipedia.org/wiki/ISO_639-1) language code and an
              [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2),
              joined by an underscore. For example: `es_MX`, meaning &quot;Spanish
              (Mexico)&quot;. Provide this parameter if you want the category strings returned
              in a particular language.<br/> _**Note**: if `locale` is not supplied, or if the
              specified language is not available, the category strings returned will be in
              the Spotify default language (American English)._

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/browse/categories",
            page=AsyncCursorURLPage[CategoryListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "locale": locale,
                        "offset": offset,
                    },
                    category_list_params.CategoryListParams,
                ),
            ),
            model=CategoryListResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def get_playlists(
        self,
        category_id: str,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CategoryGetPlaylistsResponse:
        """
        Get a list of Spotify playlists tagged with a particular category.

        Args:
          category_id: The [Spotify category ID](/documentation/web-api/concepts/spotify-uris-ids) for
              the category.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._get(
            f"/browse/categories/{category_id}/playlists",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    category_get_playlists_params.CategoryGetPlaylistsParams,
                ),
            ),
            cast_to=CategoryGetPlaylistsResponse,
        )


class CategoriesResourceWithRawResponse:
    def __init__(self, categories: CategoriesResource) -> None:
        self._categories = categories

        self.retrieve = to_raw_response_wrapper(
            categories.retrieve,
        )
        self.list = to_raw_response_wrapper(
            categories.list,
        )
        self.get_playlists = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                categories.get_playlists,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncCategoriesResourceWithRawResponse:
    def __init__(self, categories: AsyncCategoriesResource) -> None:
        self._categories = categories

        self.retrieve = async_to_raw_response_wrapper(
            categories.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            categories.list,
        )
        self.get_playlists = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                categories.get_playlists,  # pyright: ignore[reportDeprecated],
            )
        )


class CategoriesResourceWithStreamingResponse:
    def __init__(self, categories: CategoriesResource) -> None:
        self._categories = categories

        self.retrieve = to_streamed_response_wrapper(
            categories.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            categories.list,
        )
        self.get_playlists = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                categories.get_playlists,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncCategoriesResourceWithStreamingResponse:
    def __init__(self, categories: AsyncCategoriesResource) -> None:
        self._categories = categories

        self.retrieve = async_to_streamed_response_wrapper(
            categories.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            categories.list,
        )
        self.get_playlists = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                categories.get_playlists,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel
from ..shared.audiobook_base import AudiobookBase
from ..simplified_chapter_object import SimplifiedChapterObject

__all__ = ["AudiobookListResponse", "Audiobook", "AudiobookChapters"]


class AudiobookChapters(BaseModel):
    """The chapters of the audiobook."""

    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[SimplifiedChapterObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class Audiobook(AudiobookBase):
    """Information about the audiobook."""

    chapters: AudiobookChapters
    """The chapters of the audiobook."""


class AudiobookListResponse(BaseModel):
    added_at: Optional[datetime] = None
    """
    The date and time the audiobook was saved Timestamps are returned in ISO 8601
    format as Coordinated Universal Time (UTC) with a zero offset:
    YYYY-MM-DDTHH:MM:SSZ. If the time is imprecise (for example, the date/time of an
    album release), an additional field indicates the precision; see for example,
    release_date in an album object.
    """

    audiobook: Optional[Audiobook] = None
    """Information about the audiobook."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .shared.show_base import ShowBase

__all__ = ["ShowBulkRetrieveResponse"]


class ShowBulkRetrieveResponse(BaseModel):
    shows: List[ShowBase]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .me import (
    MeResource,
    AsyncMeResource,
    MeResourceWithRawResponse,
    AsyncMeResourceWithRawResponse,
    MeResourceWithStreamingResponse,
    AsyncMeResourceWithStreamingResponse,
)
from .shows import (
    ShowsResource,
    AsyncShowsResource,
    ShowsResourceWithRawResponse,
    AsyncShowsResourceWithRawResponse,
    ShowsResourceWithStreamingResponse,
    AsyncShowsResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .albums import (
    AlbumsResource,
    AsyncAlbumsResource,
    AlbumsResourceWithRawResponse,
    AsyncAlbumsResourceWithRawResponse,
    AlbumsResourceWithStreamingResponse,
    AsyncAlbumsResourceWithStreamingResponse,
)
from .browse import (
    BrowseResource,
    AsyncBrowseResource,
    BrowseResourceWithRawResponse,
    AsyncBrowseResourceWithRawResponse,
    BrowseResourceWithStreamingResponse,
    AsyncBrowseResourceWithStreamingResponse,
)
from .search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .tracks import (
    TracksResource,
    AsyncTracksResource,
    TracksResourceWithRawResponse,
    AsyncTracksResourceWithRawResponse,
    TracksResourceWithStreamingResponse,
    AsyncTracksResourceWithStreamingResponse,
)
from .artists import (
    ArtistsResource,
    AsyncArtistsResource,
    ArtistsResourceWithRawResponse,
    AsyncArtistsResourceWithRawResponse,
    ArtistsResourceWithStreamingResponse,
    AsyncArtistsResourceWithStreamingResponse,
)
from .markets import (
    MarketsResource,
    AsyncMarketsResource,
    MarketsResourceWithRawResponse,
    AsyncMarketsResourceWithRawResponse,
    MarketsResourceWithStreamingResponse,
    AsyncMarketsResourceWithStreamingResponse,
)
from .chapters import (
    ChaptersResource,
    AsyncChaptersResource,
    ChaptersResourceWithRawResponse,
    AsyncChaptersResourceWithRawResponse,
    ChaptersResourceWithStreamingResponse,
    AsyncChaptersResourceWithStreamingResponse,
)
from .episodes import (
    EpisodesResource,
    AsyncEpisodesResource,
    EpisodesResourceWithRawResponse,
    AsyncEpisodesResourceWithRawResponse,
    EpisodesResourceWithStreamingResponse,
    AsyncEpisodesResourceWithStreamingResponse,
)
from .playlists import (
    PlaylistsResource,
    AsyncPlaylistsResource,
    PlaylistsResourceWithRawResponse,
    AsyncPlaylistsResourceWithRawResponse,
    PlaylistsResourceWithStreamingResponse,
    AsyncPlaylistsResourceWithStreamingResponse,
)
from .audiobooks import (
    AudiobooksResource,
    AsyncAudiobooksResource,
    AudiobooksResourceWithRawResponse,
    AsyncAudiobooksResourceWithRawResponse,
    AudiobooksResourceWithStreamingResponse,
    AsyncAudiobooksResourceWithStreamingResponse,
)
from .audio_analysis import (
    AudioAnalysisResource,
    AsyncAudioAnalysisResource,
    AudioAnalysisResourceWithRawResponse,
    AsyncAudioAnalysisResourceWithRawResponse,
    AudioAnalysisResourceWithStreamingResponse,
    AsyncAudioAnalysisResourceWithStreamingResponse,
)
from .audio_features import (
    AudioFeaturesResource,
    AsyncAudioFeaturesResource,
    AudioFeaturesResourceWithRawResponse,
    AsyncAudioFeaturesResourceWithRawResponse,
    AudioFeaturesResourceWithStreamingResponse,
    AsyncAudioFeaturesResourceWithStreamingResponse,
)
from .recommendations import (
    RecommendationsResource,
    AsyncRecommendationsResource,
    RecommendationsResourceWithRawResponse,
    AsyncRecommendationsResourceWithRawResponse,
    RecommendationsResourceWithStreamingResponse,
    AsyncRecommendationsResourceWithStreamingResponse,
)

__all__ = [
    "AlbumsResource",
    "AsyncAlbumsResource",
    "AlbumsResourceWithRawResponse",
    "AsyncAlbumsResourceWithRawResponse",
    "AlbumsResourceWithStreamingResponse",
    "AsyncAlbumsResourceWithStreamingResponse",
    "ArtistsResource",
    "AsyncArtistsResource",
    "ArtistsResourceWithRawResponse",
    "AsyncArtistsResourceWithRawResponse",
    "ArtistsResourceWithStreamingResponse",
    "AsyncArtistsResourceWithStreamingResponse",
    "ShowsResource",
    "AsyncShowsResource",
    "ShowsResourceWithRawResponse",
    "AsyncShowsResourceWithRawResponse",
    "ShowsResourceWithStreamingResponse",
    "AsyncShowsResourceWithStreamingResponse",
    "EpisodesResource",
    "AsyncEpisodesResource",
    "EpisodesResourceWithRawResponse",
    "AsyncEpisodesResourceWithRawResponse",
    "EpisodesResourceWithStreamingResponse",
    "AsyncEpisodesResourceWithStreamingResponse",
    "AudiobooksResource",
    "AsyncAudiobooksResource",
    "AudiobooksResourceWithRawResponse",
    "AsyncAudiobooksResourceWithRawResponse",
    "AudiobooksResourceWithStreamingResponse",
    "AsyncAudiobooksResourceWithStreamingResponse",
    "MeResource",
    "AsyncMeResource",
    "MeResourceWithRawResponse",
    "AsyncMeResourceWithRawResponse",
    "MeResourceWithStreamingResponse",
    "AsyncMeResourceWithStreamingResponse",
    "ChaptersResource",
    "AsyncChaptersResource",
    "ChaptersResourceWithRawResponse",
    "AsyncChaptersResourceWithRawResponse",
    "ChaptersResourceWithStreamingResponse",
    "AsyncChaptersResourceWithStreamingResponse",
    "TracksResource",
    "AsyncTracksResource",
    "TracksResourceWithRawResponse",
    "AsyncTracksResourceWithRawResponse",
    "TracksResourceWithStreamingResponse",
    "AsyncTracksResourceWithStreamingResponse",
    "SearchResource",
    "AsyncSearchResource",
    "SearchResourceWithRawResponse",
    "AsyncSearchResourceWithRawResponse",
    "SearchResourceWithStreamingResponse",
    "AsyncSearchResourceWithStreamingResponse",
    "PlaylistsResource",
    "AsyncPlaylistsResource",
    "PlaylistsResourceWithRawResponse",
    "AsyncPlaylistsResourceWithRawResponse",
    "PlaylistsResourceWithStreamingResponse",
    "AsyncPlaylistsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "BrowseResource",
    "AsyncBrowseResource",
    "BrowseResourceWithRawResponse",
    "AsyncBrowseResourceWithRawResponse",
    "BrowseResourceWithStreamingResponse",
    "AsyncBrowseResourceWithStreamingResponse",
    "AudioFeaturesResource",
    "AsyncAudioFeaturesResource",
    "AudioFeaturesResourceWithRawResponse",
    "AsyncAudioFeaturesResourceWithRawResponse",
    "AudioFeaturesResourceWithStreamingResponse",
    "AsyncAudioFeaturesResourceWithStreamingResponse",
    "AudioAnalysisResource",
    "AsyncAudioAnalysisResource",
    "AudioAnalysisResourceWithRawResponse",
    "AsyncAudioAnalysisResourceWithRawResponse",
    "AudioAnalysisResourceWithStreamingResponse",
    "AsyncAudioAnalysisResourceWithStreamingResponse",
    "RecommendationsResource",
    "AsyncRecommendationsResource",
    "RecommendationsResourceWithRawResponse",
    "AsyncRecommendationsResourceWithRawResponse",
    "RecommendationsResourceWithStreamingResponse",
    "AsyncRecommendationsResourceWithStreamingResponse",
    "MarketsResource",
    "AsyncMarketsResource",
    "MarketsResourceWithRawResponse",
    "AsyncMarketsResourceWithRawResponse",
    "MarketsResourceWithStreamingResponse",
    "AsyncMarketsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["FollowerFollowParams"]


class FollowerFollowParams(TypedDict, total=False):
    published: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import (
    ShowBase as ShowBase,
    ImageObject as ImageObject,
    TrackObject as TrackObject,
    ArtistObject as ArtistObject,
    AuthorObject as AuthorObject,
    AudiobookBase as AudiobookBase,
    EpisodeObject as EpisodeObject,
    NarratorObject as NarratorObject,
    CopyrightObject as CopyrightObject,
    FollowersObject as FollowersObject,
    ExternalIDObject as ExternalIDObject,
    ExternalURLObject as ExternalURLObject,
    LinkedTrackObject as LinkedTrackObject,
    ResumePointObject as ResumePointObject,
    PlaylistUserObject as PlaylistUserObject,
    PlaylistTrackObject as PlaylistTrackObject,
    PagingPlaylistObject as PagingPlaylistObject,
    SimplifiedTrackObject as SimplifiedTrackObject,
    AlbumRestrictionObject as AlbumRestrictionObject,
    SimplifiedArtistObject as SimplifiedArtistObject,
    TrackRestrictionObject as TrackRestrictionObject,
    PlaylistTracksRefObject as PlaylistTracksRefObject,
    SimplifiedEpisodeObject as SimplifiedEpisodeObject,
    ChapterRestrictionObject as ChapterRestrictionObject,
    EpisodeRestrictionObject as EpisodeRestrictionObject,
    SimplifiedPlaylistObject as SimplifiedPlaylistObject,
)
from .search_query_params import SearchQueryParams as SearchQueryParams
from .market_list_response import MarketListResponse as MarketListResponse
from .me_retrieve_response import MeRetrieveResponse as MeRetrieveResponse
from .show_retrieve_params import ShowRetrieveParams as ShowRetrieveParams
from .time_interval_object import TimeIntervalObject as TimeIntervalObject
from .album_retrieve_params import AlbumRetrieveParams as AlbumRetrieveParams
from .search_query_response import SearchQueryResponse as SearchQueryResponse
from .track_retrieve_params import TrackRetrieveParams as TrackRetrieveParams
from .playlist_update_params import PlaylistUpdateParams as PlaylistUpdateParams
from .show_retrieve_response import ShowRetrieveResponse as ShowRetrieveResponse
from .album_retrieve_response import AlbumRetrieveResponse as AlbumRetrieveResponse
from .chapter_retrieve_params import ChapterRetrieveParams as ChapterRetrieveParams
from .episode_retrieve_params import EpisodeRetrieveParams as EpisodeRetrieveParams
from .album_list_tracks_params import AlbumListTracksParams as AlbumListTracksParams
from .artist_top_tracks_params import ArtistTopTracksParams as ArtistTopTracksParams
from .playlist_retrieve_params import PlaylistRetrieveParams as PlaylistRetrieveParams
from .artist_list_albums_params import ArtistListAlbumsParams as ArtistListAlbumsParams
from .audiobook_retrieve_params import AudiobookRetrieveParams as AudiobookRetrieveParams
from .chapter_retrieve_response import ChapterRetrieveResponse as ChapterRetrieveResponse
from .recommendation_get_params import RecommendationGetParams as RecommendationGetParams
from .show_bulk_retrieve_params import ShowBulkRetrieveParams as ShowBulkRetrieveParams
from .show_list_episodes_params import ShowListEpisodesParams as ShowListEpisodesParams
from .simplified_chapter_object import SimplifiedChapterObject as SimplifiedChapterObject
from .album_bulk_retrieve_params import AlbumBulkRetrieveParams as AlbumBulkRetrieveParams
from .artist_top_tracks_response import ArtistTopTracksResponse as ArtistTopTracksResponse
from .playlist_retrieve_response import PlaylistRetrieveResponse as PlaylistRetrieveResponse
from .track_bulk_retrieve_params import TrackBulkRetrieveParams as TrackBulkRetrieveParams
from .artist_bulk_retrieve_params import ArtistBulkRetrieveParams as ArtistBulkRetrieveParams
from .artist_list_albums_response import ArtistListAlbumsResponse as ArtistListAlbumsResponse
from .audiobook_retrieve_response import AudiobookRetrieveResponse as AudiobookRetrieveResponse
from .recommendation_get_response import RecommendationGetResponse as RecommendationGetResponse
from .show_bulk_retrieve_response import ShowBulkRetrieveResponse as ShowBulkRetrieveResponse
from .album_bulk_retrieve_response import AlbumBulkRetrieveResponse as AlbumBulkRetrieveResponse
from .chapter_bulk_retrieve_params import ChapterBulkRetrieveParams as ChapterBulkRetrieveParams
from .episode_bulk_retrieve_params import EpisodeBulkRetrieveParams as EpisodeBulkRetrieveParams
from .track_bulk_retrieve_response import TrackBulkRetrieveResponse as TrackBulkRetrieveResponse
from .artist_bulk_retrieve_response import ArtistBulkRetrieveResponse as ArtistBulkRetrieveResponse
from .audiobook_bulk_retrieve_params import AudiobookBulkRetrieveParams as AudiobookBulkRetrieveParams
from .audiobook_list_chapters_params import AudiobookListChaptersParams as AudiobookListChaptersParams
from .browse_get_new_releases_params import BrowseGetNewReleasesParams as BrowseGetNewReleasesParams
from .chapter_bulk_retrieve_response import ChapterBulkRetrieveResponse as ChapterBulkRetrieveResponse
from .episode_bulk_retrieve_response import EpisodeBulkRetrieveResponse as EpisodeBulkRetrieveResponse
from .user_retrieve_profile_response import UserRetrieveProfileResponse as UserRetrieveProfileResponse
from .audio_feature_retrieve_response import AudioFeatureRetrieveResponse as AudioFeatureRetrieveResponse
from .audio_analysis_retrieve_response import AudioAnalysisRetrieveResponse as AudioAnalysisRetrieveResponse
from .audiobook_bulk_retrieve_response import AudiobookBulkRetrieveResponse as AudiobookBulkRetrieveResponse
from .browse_get_new_releases_response import BrowseGetNewReleasesResponse as BrowseGetNewReleasesResponse
from .audio_feature_bulk_retrieve_params import AudioFeatureBulkRetrieveParams as AudioFeatureBulkRetrieveParams
from .artist_list_related_artists_response import ArtistListRelatedArtistsResponse as ArtistListRelatedArtistsResponse
from .audio_feature_bulk_retrieve_response import AudioFeatureBulkRetrieveResponse as AudioFeatureBulkRetrieveResponse
from .browse_get_featured_playlists_params import BrowseGetFeaturedPlaylistsParams as BrowseGetFeaturedPlaylistsParams
from .browse_get_featured_playlists_response import (
    BrowseGetFeaturedPlaylistsResponse as BrowseGetFeaturedPlaylistsResponse,
)
from .recommendation_list_available_genre_seeds_response import (
    RecommendationListAvailableGenreSeedsResponse as RecommendationListAvailableGenreSeedsResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["FollowersObject"]


class FollowersObject(BaseModel):
    href: Optional[str] = None
    """
    This will always be set to null, as the Web API does not support it at the
    moment.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    total: Optional[int] = None
    """The total number of followers."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AlbumBulkRetrieveParams"]


class AlbumBulkRetrieveParams(TypedDict, total=False):
    ids: Required[str]
    """
    A comma-separated list of the
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the albums.
    Maximum: 20 IDs.
    """

    market: str
    """
    An
    [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    If a country code is specified, only content that is available in that market
    will be returned.<br/> If a valid user access token is specified in the request
    header, the country associated with the user account will take priority over
    this parameter.<br/> _**Note**: If neither market or user country are provided,
    the content is considered unavailable for the client._<br/> Users can view the
    country that is associated with their account in the
    [account settings](https://www.spotify.com/account/overview/).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .external_url_object import ExternalURLObject

__all__ = ["PlaylistUserObject"]


class PlaylistUserObject(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids) for this
    user.
    """

    external_urls: Optional[ExternalURLObject] = None
    """Known public external URLs for this user."""

    href: Optional[str] = None
    """A link to the Web API endpoint for this user."""

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    type: Optional[Literal["user"]] = None
    """The object type."""

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for this
    user.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AudioFeatureBulkRetrieveResponse", "AudioFeature"]


class AudioFeature(BaseModel):
    id: Optional[str] = None
    """The Spotify ID for the track."""

    acousticness: Optional[float] = None
    """A confidence measure from 0.0 to 1.0 of whether the track is acoustic.

    1.0 represents high confidence the track is acoustic.
    """

    analysis_url: Optional[str] = None
    """A URL to access the full audio analysis of this track.

    An access token is required to access this data.
    """

    danceability: Optional[float] = None
    """
    Danceability describes how suitable a track is for dancing based on a
    combination of musical elements including tempo, rhythm stability, beat
    strength, and overall regularity. A value of 0.0 is least danceable and 1.0 is
    most danceable.
    """

    duration_ms: Optional[int] = None
    """The duration of the track in milliseconds."""

    energy: Optional[float] = None
    """
    Energy is a measure from 0.0 to 1.0 and represents a perceptual measure of
    intensity and activity. Typically, energetic tracks feel fast, loud, and noisy.
    For example, death metal has high energy, while a Bach prelude scores low on the
    scale. Perceptual features contributing to this attribute include dynamic range,
    perceived loudness, timbre, onset rate, and general entropy.
    """

    instrumentalness: Optional[float] = None
    """Predicts whether a track contains no vocals.

    "Ooh" and "aah" sounds are treated as instrumental in this context. Rap or
    spoken word tracks are clearly "vocal". The closer the instrumentalness value is
    to 1.0, the greater likelihood the track contains no vocal content. Values above
    0.5 are intended to represent instrumental tracks, but confidence is higher as
    the value approaches 1.0.
    """

    key: Optional[int] = None
    """The key the track is in.

    Integers map to pitches using standard
    [Pitch Class notation](https://en.wikipedia.org/wiki/Pitch_class). E.g. 0 = C, 1
    = C♯/D♭, 2 = D, and so on. If no key was detected, the value is -1.
    """

    liveness: Optional[float] = None
    """Detects the presence of an audience in the recording.

    Higher liveness values represent an increased probability that the track was
    performed live. A value above 0.8 provides strong likelihood that the track is
    live.
    """

    loudness: Optional[float] = None
    """The overall loudness of a track in decibels (dB).

    Loudness values are averaged across the entire track and are useful for
    comparing relative loudness of tracks. Loudness is the quality of a sound that
    is the primary psychological correlate of physical strength (amplitude). Values
    typically range between -60 and 0 db.
    """

    mode: Optional[int] = None
    """
    Mode indicates the modality (major or minor) of a track, the type of scale from
    which its melodic content is derived. Major is represented by 1 and minor is 0.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    speechiness: Optional[float] = None
    """Speechiness detects the presence of spoken words in a track.

    The more exclusively speech-like the recording (e.g. talk show, audio book,
    poetry), the closer to 1.0 the attribute value. Values above 0.66 describe
    tracks that are probably made entirely of spoken words. Values between 0.33 and
    0.66 describe tracks that may contain both music and speech, either in sections
    or layered, including such cases as rap music. Values below 0.33 most likely
    represent music and other non-speech-like tracks.
    """

    tempo: Optional[float] = None
    """The overall estimated tempo of a track in beats per minute (BPM).

    In musical terminology, tempo is the speed or pace of a given piece and derives
    directly from the average beat duration.
    """

    time_signature: Optional[int] = None
    """An estimated time signature.

    The time signature (meter) is a notational convention to specify how many beats
    are in each bar (or measure). The time signature ranges from 3 to 7 indicating
    time signatures of "3/4", to "7/4".
    """

    track_href: Optional[str] = None
    """A link to the Web API endpoint providing full details of the track."""

    type: Optional[Literal["audio_features"]] = None
    """The object type."""

    uri: Optional[str] = None
    """The Spotify URI for the track."""

    valence: Optional[float] = None
    """
    A measure from 0.0 to 1.0 describing the musical positiveness conveyed by a
    track. Tracks with high valence sound more positive (e.g. happy, cheerful,
    euphoric), while tracks with low valence sound more negative (e.g. sad,
    depressed, angry).
    """


class AudioFeatureBulkRetrieveResponse(BaseModel):
    audio_features: List[AudioFeature]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .shared.artist_object import ArtistObject

__all__ = ["ArtistListRelatedArtistsResponse"]


class ArtistListRelatedArtistsResponse(BaseModel):
    artists: List[ArtistObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .playlist_list_params import PlaylistListParams as PlaylistListParams
from .playlist_create_params import PlaylistCreateParams as PlaylistCreateParams
from .playlist_create_response import PlaylistCreateResponse as PlaylistCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .shows import (
    ShowsResource,
    AsyncShowsResource,
    ShowsResourceWithRawResponse,
    AsyncShowsResourceWithRawResponse,
    ShowsResourceWithStreamingResponse,
    AsyncShowsResourceWithStreamingResponse,
)
from .albums import (
    AlbumsResource,
    AsyncAlbumsResource,
    AlbumsResourceWithRawResponse,
    AsyncAlbumsResourceWithRawResponse,
    AlbumsResourceWithStreamingResponse,
    AsyncAlbumsResourceWithStreamingResponse,
)
from .tracks import (
    TracksResource,
    AsyncTracksResource,
    TracksResourceWithRawResponse,
    AsyncTracksResourceWithRawResponse,
    TracksResourceWithStreamingResponse,
    AsyncTracksResourceWithStreamingResponse,
)
from ..._types import Body, Query, Headers, NotGiven, not_given
from .episodes import (
    EpisodesResource,
    AsyncEpisodesResource,
    EpisodesResourceWithRawResponse,
    AsyncEpisodesResourceWithRawResponse,
    EpisodesResourceWithStreamingResponse,
    AsyncEpisodesResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .following import (
    FollowingResource,
    AsyncFollowingResource,
    FollowingResourceWithRawResponse,
    AsyncFollowingResourceWithRawResponse,
    FollowingResourceWithStreamingResponse,
    AsyncFollowingResourceWithStreamingResponse,
)
from .playlists import (
    PlaylistsResource,
    AsyncPlaylistsResource,
    PlaylistsResourceWithRawResponse,
    AsyncPlaylistsResourceWithRawResponse,
    PlaylistsResourceWithStreamingResponse,
    AsyncPlaylistsResourceWithStreamingResponse,
)
from .audiobooks import (
    AudiobooksResource,
    AsyncAudiobooksResource,
    AudiobooksResourceWithRawResponse,
    AsyncAudiobooksResourceWithRawResponse,
    AudiobooksResourceWithStreamingResponse,
    AsyncAudiobooksResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .player.player import (
    PlayerResource,
    AsyncPlayerResource,
    PlayerResourceWithRawResponse,
    AsyncPlayerResourceWithRawResponse,
    PlayerResourceWithStreamingResponse,
    AsyncPlayerResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.me_retrieve_response import MeRetrieveResponse

__all__ = ["MeResource", "AsyncMeResource"]


class MeResource(SyncAPIResource):
    @cached_property
    def audiobooks(self) -> AudiobooksResource:
        return AudiobooksResource(self._client)

    @cached_property
    def playlists(self) -> PlaylistsResource:
        return PlaylistsResource(self._client)

    @cached_property
    def top(self) -> TopResource:
        return TopResource(self._client)

    @cached_property
    def albums(self) -> AlbumsResource:
        return AlbumsResource(self._client)

    @cached_property
    def tracks(self) -> TracksResource:
        return TracksResource(self._client)

    @cached_property
    def episodes(self) -> EpisodesResource:
        return EpisodesResource(self._client)

    @cached_property
    def shows(self) -> ShowsResource:
        return ShowsResource(self._client)

    @cached_property
    def following(self) -> FollowingResource:
        return FollowingResource(self._client)

    @cached_property
    def player(self) -> PlayerResource:
        return PlayerResource(self._client)

    @cached_property
    def with_raw_response(self) -> MeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return MeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return MeResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeRetrieveResponse:
        """
        Get detailed profile information about the current user (including the current
        user's username).
        """
        return self._get(
            "/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeRetrieveResponse,
        )


class AsyncMeResource(AsyncAPIResource):
    @cached_property
    def audiobooks(self) -> AsyncAudiobooksResource:
        return AsyncAudiobooksResource(self._client)

    @cached_property
    def playlists(self) -> AsyncPlaylistsResource:
        return AsyncPlaylistsResource(self._client)

    @cached_property
    def top(self) -> AsyncTopResource:
        return AsyncTopResource(self._client)

    @cached_property
    def albums(self) -> AsyncAlbumsResource:
        return AsyncAlbumsResource(self._client)

    @cached_property
    def tracks(self) -> AsyncTracksResource:
        return AsyncTracksResource(self._client)

    @cached_property
    def episodes(self) -> AsyncEpisodesResource:
        return AsyncEpisodesResource(self._client)

    @cached_property
    def shows(self) -> AsyncShowsResource:
        return AsyncShowsResource(self._client)

    @cached_property
    def following(self) -> AsyncFollowingResource:
        return AsyncFollowingResource(self._client)

    @cached_property
    def player(self) -> AsyncPlayerResource:
        return AsyncPlayerResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncMeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncMeResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeRetrieveResponse:
        """
        Get detailed profile information about the current user (including the current
        user's username).
        """
        return await self._get(
            "/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeRetrieveResponse,
        )


class MeResourceWithRawResponse:
    def __init__(self, me: MeResource) -> None:
        self._me = me

        self.retrieve = to_raw_response_wrapper(
            me.retrieve,
        )

    @cached_property
    def audiobooks(self) -> AudiobooksResourceWithRawResponse:
        return AudiobooksResourceWithRawResponse(self._me.audiobooks)

    @cached_property
    def playlists(self) -> PlaylistsResourceWithRawResponse:
        return PlaylistsResourceWithRawResponse(self._me.playlists)

    @cached_property
    def top(self) -> TopResourceWithRawResponse:
        return TopResourceWithRawResponse(self._me.top)

    @cached_property
    def albums(self) -> AlbumsResourceWithRawResponse:
        return AlbumsResourceWithRawResponse(self._me.albums)

    @cached_property
    def tracks(self) -> TracksResourceWithRawResponse:
        return TracksResourceWithRawResponse(self._me.tracks)

    @cached_property
    def episodes(self) -> EpisodesResourceWithRawResponse:
        return EpisodesResourceWithRawResponse(self._me.episodes)

    @cached_property
    def shows(self) -> ShowsResourceWithRawResponse:
        return ShowsResourceWithRawResponse(self._me.shows)

    @cached_property
    def following(self) -> FollowingResourceWithRawResponse:
        return FollowingResourceWithRawResponse(self._me.following)

    @cached_property
    def player(self) -> PlayerResourceWithRawResponse:
        return PlayerResourceWithRawResponse(self._me.player)


class AsyncMeResourceWithRawResponse:
    def __init__(self, me: AsyncMeResource) -> None:
        self._me = me

        self.retrieve = async_to_raw_response_wrapper(
            me.retrieve,
        )

    @cached_property
    def audiobooks(self) -> AsyncAudiobooksResourceWithRawResponse:
        return AsyncAudiobooksResourceWithRawResponse(self._me.audiobooks)

    @cached_property
    def playlists(self) -> AsyncPlaylistsResourceWithRawResponse:
        return AsyncPlaylistsResourceWithRawResponse(self._me.playlists)

    @cached_property
    def top(self) -> AsyncTopResourceWithRawResponse:
        return AsyncTopResourceWithRawResponse(self._me.top)

    @cached_property
    def albums(self) -> AsyncAlbumsResourceWithRawResponse:
        return AsyncAlbumsResourceWithRawResponse(self._me.albums)

    @cached_property
    def tracks(self) -> AsyncTracksResourceWithRawResponse:
        return AsyncTracksResourceWithRawResponse(self._me.tracks)

    @cached_property
    def episodes(self) -> AsyncEpisodesResourceWithRawResponse:
        return AsyncEpisodesResourceWithRawResponse(self._me.episodes)

    @cached_property
    def shows(self) -> AsyncShowsResourceWithRawResponse:
        return AsyncShowsResourceWithRawResponse(self._me.shows)

    @cached_property
    def following(self) -> AsyncFollowingResourceWithRawResponse:
        return AsyncFollowingResourceWithRawResponse(self._me.following)

    @cached_property
    def player(self) -> AsyncPlayerResourceWithRawResponse:
        return AsyncPlayerResourceWithRawResponse(self._me.player)


class MeResourceWithStreamingResponse:
    def __init__(self, me: MeResource) -> None:
        self._me = me

        self.retrieve = to_streamed_response_wrapper(
            me.retrieve,
        )

    @cached_property
    def audiobooks(self) -> AudiobooksResourceWithStreamingResponse:
        return AudiobooksResourceWithStreamingResponse(self._me.audiobooks)

    @cached_property
    def playlists(self) -> PlaylistsResourceWithStreamingResponse:
        return PlaylistsResourceWithStreamingResponse(self._me.playlists)

    @cached_property
    def top(self) -> TopResourceWithStreamingResponse:
        return TopResourceWithStreamingResponse(self._me.top)

    @cached_property
    def albums(self) -> AlbumsResourceWithStreamingResponse:
        return AlbumsResourceWithStreamingResponse(self._me.albums)

    @cached_property
    def tracks(self) -> TracksResourceWithStreamingResponse:
        return TracksResourceWithStreamingResponse(self._me.tracks)

    @cached_property
    def episodes(self) -> EpisodesResourceWithStreamingResponse:
        return EpisodesResourceWithStreamingResponse(self._me.episodes)

    @cached_property
    def shows(self) -> ShowsResourceWithStreamingResponse:
        return ShowsResourceWithStreamingResponse(self._me.shows)

    @cached_property
    def following(self) -> FollowingResourceWithStreamingResponse:
        return FollowingResourceWithStreamingResponse(self._me.following)

    @cached_property
    def player(self) -> PlayerResourceWithStreamingResponse:
        return PlayerResourceWithStreamingResponse(self._me.player)


class AsyncMeResourceWithStreamingResponse:
    def __init__(self, me: AsyncMeResource) -> None:
        self._me = me

        self.retrieve = async_to_streamed_response_wrapper(
            me.retrieve,
        )

    @cached_property
    def audiobooks(self) -> AsyncAudiobooksResourceWithStreamingResponse:
        return AsyncAudiobooksResourceWithStreamingResponse(self._me.audiobooks)

    @cached_property
    def playlists(self) -> AsyncPlaylistsResourceWithStreamingResponse:
        return AsyncPlaylistsResourceWithStreamingResponse(self._me.playlists)

    @cached_property
    def top(self) -> AsyncTopResourceWithStreamingResponse:
        return AsyncTopResourceWithStreamingResponse(self._me.top)

    @cached_property
    def albums(self) -> AsyncAlbumsResourceWithStreamingResponse:
        return AsyncAlbumsResourceWithStreamingResponse(self._me.albums)

    @cached_property
    def tracks(self) -> AsyncTracksResourceWithStreamingResponse:
        return AsyncTracksResourceWithStreamingResponse(self._me.tracks)

    @cached_property
    def episodes(self) -> AsyncEpisodesResourceWithStreamingResponse:
        return AsyncEpisodesResourceWithStreamingResponse(self._me.episodes)

    @cached_property
    def shows(self) -> AsyncShowsResourceWithStreamingResponse:
        return AsyncShowsResourceWithStreamingResponse(self._me.shows)

    @cached_property
    def following(self) -> AsyncFollowingResourceWithStreamingResponse:
        return AsyncFollowingResourceWithStreamingResponse(self._me.following)

    @cached_property
    def player(self) -> AsyncPlayerResourceWithStreamingResponse:
        return AsyncPlayerResourceWithStreamingResponse(self._me.player)
